@@ -2662,6 +2662,10 @@ removeDeadKeys(db_connection_t *dbconn, key_data_t** keylist,
 	key_dependency_free(deplist2[i]);
     }
 	free(deplist2);
+        
+    int deleteCount = hsm_key_factory_delete_key(dbconn);
+    ods_log_info("[%s] %s: keys deleted from HSM: %d", module_str, scmd, deleteCount);
+
 	return first_purge;
 }
 
@@ -2790,11 +2794,6 @@ update(engine_type *engine, db_connection_t *dbconn, zone_db_t *zone, policy_t c
     }
     key_data_list_free(key_list);
 
-    /*
-     * Update zone.
-     */
-    zone_return_time = updateZone(dbconn, policy, zone, now, allow_unsigned, zone_updated,
-	    keylist, keylist_size, deplist);
 
     /*
      * Only purge old keys if the policy says so.
@@ -2803,6 +2802,14 @@ update(engine_type *engine, db_connection_t *dbconn, zone_db_t *zone, policy_t c
 	    purge_return_time = removeDeadKeys(dbconn, keylist, keylist_size, deplist, now,
 	        policy_keys_purge_after(policy));
 	}
+    
+    
+    /*
+     * Update zone.
+     */
+    zone_return_time = updateZone(dbconn, policy, zone, now, allow_unsigned, zone_updated,
+	    keylist, keylist_size, deplist);
+
 
     /*
      * Always set these flags. Normally this needs to be done _only_ when the
@@ -2876,7 +2883,7 @@ update(engine_type *engine, db_connection_t *dbconn, zone_db_t *zone, policy_t c
             }
         }
 	}
-
+            
     /*
      * Release cached objects.
      */
