@@ -30,7 +30,7 @@
 #ifndef _KEYSTATE_DS_RETRACT_TASK_H_
 #define _KEYSTATE_DS_RETRACT_TASK_H_
 
-task_type *
+extern task_type *
 keystate_ds_retract_task(engine_type *engine, char const *owner);
 
 #endif /*_KEYSTATE_DS_RETRACT_TASK_H_*/
