@@ -26,8 +26,10 @@
  */
 
 #include "config.h"
+#include <getopt.h>
 
-#include "daemon/cmdhandler.h"
+#include "cmdhandler.h"
+#include "daemon/enforcercommands.h"
 #include "daemon/engine.h"
 #include "file.h"
 #include "log.h"
@@ -65,7 +67,7 @@ perform_hsmkey_import(int sockfd, db_connection_t *dbconn,
     hsm_key_t *hsm_key = NULL;
     char *hsm_err;
     libhsm_key_t *libhsmkey;
-    zone_t *zone;
+    zone_db_t *zone;
 	
   /* Create an HSM context and check that the repository exists  */
     if (!(hsm_ctx = hsm_create_context())) {
@@ -91,7 +93,7 @@ perform_hsmkey_import(int sockfd, db_connection_t *dbconn,
 	hsm_destroy_context(hsm_ctx);
 	return -1;
     }
-    free(libhsmkey);
+    libhsm_key_free(libhsmkey);
     hsm_key = hsm_key_new_get_by_locator(dbconn, ckaid);
     if (hsm_key) {
         ods_log_error("[%s] Error: Already used this key with this locator: %s", module_str, ckaid);
@@ -101,14 +103,14 @@ perform_hsmkey_import(int sockfd, db_connection_t *dbconn,
         return -1;
     }
 
-    zone = zone_new_get_by_name(dbconn, zonename);
+    zone = zone_db_new_get_by_name(dbconn, zonename);
     if (!(hsm_key = hsm_key_new(dbconn))
                 || hsm_key_set_algorithm(hsm_key, alg)
                 || hsm_key_set_bits(hsm_key, bits)
                 || hsm_key_set_inception(hsm_key, time)
                 || hsm_key_set_key_type(hsm_key, HSM_KEY_KEY_TYPE_RSA)
                 || hsm_key_set_locator(hsm_key, ckaid)
-                || hsm_key_set_policy_id(hsm_key, zone_policy_id(zone))
+                || hsm_key_set_policy_id(hsm_key, zone_db_policy_id(zone))
                 || hsm_key_set_repository(hsm_key, rep)
                 || hsm_key_set_role(hsm_key, keytype)
                 || hsm_key_set_state(hsm_key, (hsm_key_state_t)HSM_KEY_STATE_PRIVATE)
@@ -117,13 +119,13 @@ perform_hsmkey_import(int sockfd, db_connection_t *dbconn,
         ods_log_error("[%s] hsm key creation failed, database or memory error", module_str);
         hsm_key_free(hsm_key);                
         hsm_destroy_context(hsm_ctx);
-        zone_free(zone);
+        zone_db_free(zone);
         return -1;
     }
     ods_log_debug("[%s] hsm key with this locator %s is created successfully", module_str, ckaid);
     hsm_key_free(hsm_key);
     hsm_destroy_context(hsm_ctx);
-    zone_free(zone);
+    zone_db_free(zone);
     return 0;
 }
 
@@ -138,7 +140,7 @@ perform_keydata_import(int sockfd, db_connection_t *dbconn,
     uint16_t tag;
     hsm_key_t * hsmkey;
     libhsm_key_t *libhsmkey;
-    zone_t *zone;
+    zone_db_t *zone;
 
     /* Create a HSM context and check that the repository exists  */
     if (!(hsm_ctx = hsm_create_context())) {
@@ -164,19 +166,19 @@ perform_keydata_import(int sockfd, db_connection_t *dbconn,
         hsm_destroy_context(hsm_ctx);
         return -1;
     }
-    free(libhsmkey);
+    libhsm_key_free(libhsmkey);
     if (!(hsmkey = hsm_key_new_get_by_locator(dbconn, ckaid))) {
         ods_log_error("[%s] Error: Cannot get hsmkey %s from database, database error", module_str, ckaid);
         hsm_destroy_context(hsm_ctx);
         return -1;
     }
-    if (hsm_keytag(ckaid, alg, keytype == 1 ? 1 : 0, &tag)) {
+    if (hsm_keytag(ckaid, alg, KEY_DATA_ROLE_SEP(keytype), &tag)) {
         ods_log_error("[%s] Error: Keytag for this key %s is not correct", module_str, ckaid);
     }
 
-    zone = zone_new_get_by_name(dbconn, zonename);
+    zone = zone_db_new_get_by_name(dbconn, zonename);
     if (!(key_data = key_data_new(dbconn))
-                || key_data_set_zone_id(key_data, zone_id(zone))
+                || key_data_set_zone_id(key_data, zone_db_id(zone))
                 || key_data_set_hsm_key_id(key_data, hsm_key_id(hsmkey))
 		|| key_data_set_algorithm (key_data, alg)
                 || key_data_set_inception(key_data, time)
@@ -194,10 +196,10 @@ perform_keydata_import(int sockfd, db_connection_t *dbconn,
         hsm_key_free(hsmkey);
         key_data_free(key_data);
         hsm_destroy_context(hsm_ctx);
-        zone_free(zone);
+        zone_db_free(zone);
         return -1;
     }
-    zone_free(zone);
+    zone_db_free(zone);
     ods_log_debug("[%s] key data with this locator %s is created successfully", module_str, ckaid);
     key_data_free(key_data);
     hsm_destroy_context(hsm_ctx);
@@ -219,7 +221,7 @@ perform_keystate_import(int sockfd, db_connection_t *dbconn,
     const db_value_t* keydataid;
     policy_t* policy;
     libhsm_key_t *libhsmkey;
-    zone_t *zone;
+    zone_db_t *zone;
 
     /* Create a HSM context and check that the repository exists  */
     if (!(hsm_ctx = hsm_create_context())) {
@@ -245,14 +247,14 @@ perform_keystate_import(int sockfd, db_connection_t *dbconn,
         hsm_destroy_context(hsm_ctx);
         return -1;
     }
-    free(libhsmkey);
+    libhsm_key_free(libhsmkey);
     key = key_data_new_get_by_hsm_key_id(dbconn, hsmkeyid);
     keydataid = key_data_id(key);
 
     policy = policy_new(dbconn);
-    zone = zone_new_get_by_name(dbconn, zonename);
-    policy_get_by_id(policy, zone_policy_id(zone));
-    zone_free(zone);
+    zone = zone_db_new_get_by_name(dbconn, zonename);
+    policy_get_by_id(policy, zone_db_policy_id(zone));
+    zone_db_free(zone);
 
     if (!(key_state = key_state_new(dbconn))
                 || key_state_set_key_data_id(key_state, keydataid)
@@ -376,19 +378,12 @@ help(int sockfd)
 }
 
 static int
-handles(const char *cmd, ssize_t n)
+run(int sockfd, cmdhandler_ctx_type* context, const char *cmd)
 {
-    return ods_check_command(cmd, n, key_import_funcblock()->cmdname)?1:0;
-}
-
-static int
-run(int sockfd, engine_type* engine, const char *cmd, ssize_t n,
-	db_connection_t *dbconn)
-{
-    #define NARGV 16
+    #define NARGV 18
     char buf[ODS_SE_MAXLINE];
     const char *argv[NARGV];
-    int argc;
+    int argc = 0, long_index = 0, opt = 0;
     const char *ckaid = NULL;
     const char *repository = NULL;
     const char *zonename = NULL;
@@ -397,16 +392,27 @@ run(int sockfd, engine_type* engine, const char *cmd, ssize_t n,
     const char* keytype = NULL;
     const char* keystate = NULL;
     const char *time = NULL;
-    zone_t *zone = NULL;
+    zone_db_t *zone = NULL;
     time_t inception = 0;
     struct tm tm;
-    (void)engine;
     int setmin;
     db_value_t *hsmkey_id;
     policy_key_t *policy_key;
+    db_connection_t* dbconn = getconnectioncontext(context);
+
+    static struct option long_options[] = {
+        {"zone", required_argument, 0, 'z'},
+        {"cka_id", required_argument, 0, 'k'},
+        {"repository", required_argument, 0, 'r'},
+        {"bits", required_argument, 0, 'b'},
+        {"algorithm", required_argument, 0, 'g'},
+        {"keytype", required_argument, 0, 't'},
+        {"keystate", required_argument, 0, 'e'},
+        {"inception_time", required_argument, 0, 'w'},
+        {0, 0, 0, 0}
+    };
 	
-    ods_log_debug("[%s] %s command", module_str, key_import_funcblock()->cmdname);
-    cmd = ods_check_command(cmd, n, key_import_funcblock()->cmdname);
+    ods_log_debug("[%s] %s command", module_str, key_import_funcblock.cmdname);
 	
     /* Use buf as an intermediate buffer for the command.*/
     strncpy(buf, cmd, sizeof(buf));
@@ -414,20 +420,47 @@ run(int sockfd, engine_type* engine, const char *cmd, ssize_t n,
 	
     /* separate the arguments*/
     argc = ods_str_explode(buf, NARGV, argv);
-    if (argc > NARGV) {
-        ods_log_error("[%s] too many arguments for %s command", module_str, key_import_funcblock()->cmdname);
-        client_printf_err(sockfd,"too many arguments\n");
+    if (argc == -1) {
+        client_printf_err(sockfd, "too many arguments\n");
+        ods_log_error("[%s] too many arguments for %s command",
+                      module_str, key_import_funcblock.cmdname);
         return -1;
     }
-	
-    (void)ods_find_arg_and_param(&argc,argv, "cka_id","k",&ckaid);
-    (void)ods_find_arg_and_param(&argc,argv, "repository","r",&repository);
-    (void)ods_find_arg_and_param(&argc,argv, "bits","b",&bits);
-    (void)ods_find_arg_and_param(&argc,argv, "algorithm","g",&algorithm);
-    (void)ods_find_arg_and_param(&argc,argv,"zone","z",&zonename);
-    (void)ods_find_arg_and_param(&argc, argv, "keytype", "t", &keytype);
-    (void)ods_find_arg_and_param(&argc, argv, "keystate", "e", &keystate);
-    (void)ods_find_arg_and_param(&argc,argv, "inception_time","w",&time);
+
+    optind = 0;
+    while ((opt = getopt_long(argc, (char* const*)argv, "z:k:r:b:g:t:e:w:", long_options, &long_index)) != -1) {
+        switch (opt) {
+            case 'z':
+                zonename = optarg;
+                break;
+            case 'k':
+                ckaid = optarg;
+                break;
+            case 'r':
+                repository = optarg;
+                break;
+            case 'b':
+                bits = optarg;
+                break;
+            case 'g':
+                algorithm = optarg;
+                break;
+            case 't':
+                keytype = optarg;
+                break;
+            case 'e':
+                keystate = optarg;
+                break;
+            case 'w':
+                time = optarg;
+                break;
+            default:
+                client_printf_err(sockfd, "unknown arguments\n");
+                ods_log_error("[%s] unknown arguments for %s command",
+                                module_str, key_import_funcblock.cmdname);
+                return -1;
+        }
+    }
 
     if (keytype) {
         if (strcasecmp(keytype, "KSK") && strcasecmp(keytype, "ZSK") && strcasecmp(keytype, "CSK")) {
@@ -436,38 +469,63 @@ run(int sockfd, engine_type* engine, const char *cmd, ssize_t n,
             return -1;
         }	
     }
+    else {
+        ods_log_error("[%s] specify keytype for command %s", module_str, cmd);
+        client_printf_err(sockfd, "specify keytype: ZSK, KSK or CSK\n");
+        return -1;
+    }
 
     if (keystate) {
         if (strcasecmp(keystate, "generate") && strcasecmp(keystate, "publish") && strcasecmp(keystate, "ready") && strcasecmp(keystate, "active") && strcasecmp(keystate, "retire") && strcasecmp(keystate, "revoke")) {
             ods_log_error("[%s] unknown keystate", module_str);
-            client_printf_err(sockfd, "unknown keystate\n");
+            client_printf_err(sockfd, "unknown keystate: states are generate, publish, ready, active or retire\n");
             return -1;
         } 
     }
-
-    if (argc) {
-        ods_log_error("[%s] unknown arguments for %s command", module_str, key_import_funcblock()->cmdname);
-        client_printf_err(sockfd,"unknown arguments\n");
+    else {
+        ods_log_error("[%s] specify keystate for command %s", module_str, cmd);
+        client_printf_err(sockfd, "specify keystate: generate, publish, ready, active or retire\n");
         return -1;
     }
 
     if (!zonename) {
-        ods_log_error("[%s] expected --zone for %s command", module_str, key_import_funcblock()->cmdname);
+        ods_log_error("[%s] expected --zone for %s command", module_str, key_import_funcblock.cmdname);
         client_printf_err(sockfd, "expected --zone \n");
         return -1;
     }
-    if (zonename && !(zone = zone_new_get_by_name(dbconn, zonename))) {
+    if (zonename && !(zone = zone_db_new_get_by_name(dbconn, zonename))) {
         ods_log_error("[%s] Unknown zone: %s", module_str, zonename);
         client_printf_err(sockfd, "Unknown zone: %s\n", zonename);
         return -1;
     }
     free(zone);
     zone = NULL;
-	
-    if (strptime(time, "%Y-%m-%d-%H:%M:%S", &tm)) {
+
+    if (!algorithm) {
+        ods_log_error("[%s] specify an algorithm for command %s", module_str, cmd);
+        client_printf_err(sockfd, "specify an algorithm\n");
+        return -1;
+    }
+    if (!bits) {
+        ods_log_error("[%s] specify bits for command %s", module_str, cmd);
+        client_printf_err(sockfd, "specify bits\n");
+        return -1;
+    }
+    if (!repository) {
+        ods_log_error("[%s] specify repository for command %s", module_str, cmd);
+        client_printf_err(sockfd, "specify repository \n");
+        return -1;
+    }
+
+    if (time && strptime(time, "%Y-%m-%d-%H:%M:%S", &tm)) {
         tm.tm_isdst = -1;
         inception = mktime(&tm);
     }
+    else {
+        ods_log_error("[%s] specify inception time for command %s", module_str, cmd);
+        client_printf_err(sockfd, "specify inception time YYYY-MM-DD-HH:MM:SS\n");
+        return -1;
+    }
 
     /* gen = 0, pub = 1, ready = 2, act = 3, ... */
     int state = -1;
@@ -493,15 +551,16 @@ run(int sockfd, engine_type* engine, const char *cmd, ssize_t n,
         type = 3;
 
     hsmkey_id = db_value_new();
-    zone = zone_new_get_by_name(dbconn, zonename);
-    policy_key = policy_key_new_get_by_policyid_and_role(dbconn, zone_policy_id(zone), type);
-    zone_free(zone);
+    zone = zone_db_new_get_by_name(dbconn, zonename);
+    policy_key = policy_key_new_get_by_policyid_and_role(dbconn, zone_db_policy_id(zone), type);
+    zone_db_free(zone);
     if (!policy_key) {
         ods_log_error("Unable to get policyKey, database error!");
         client_printf_err(sockfd, "Unable to get policyKey, database error!\n");
         db_value_free((void*)hsmkey_id);
         return -1;
     }
+
     if (atoi(algorithm) != policy_key_algorithm(policy_key)) {
         ods_log_error("Error: the given algorithm in import command doesn't match the algorithm in kasp");
         client_printf_err(sockfd, "The given algorithm doesn't match the algorithm in kasp\n");
@@ -526,13 +585,6 @@ run(int sockfd, engine_type* engine, const char *cmd, ssize_t n,
     return 0;
 }
 
-static struct cmd_func_block funcblock = {
-    "key import", &usage, &help, &handles, &run
+struct cmd_func_block key_import_funcblock = {
+    "key import", &usage, &help, NULL, &run
 };
-
-struct cmd_func_block*
-key_import_funcblock(void)
-{
-    return &funcblock;
-}
-
