@@ -31,7 +31,7 @@
 #include "db/policy.h"
 #include "db/policy_key.h"
 #include "utils/kc_helper.h"
-#include "db/zone.h"
+#include "db/zone_db.h"
 #include "db/hsm_key.h"
 #include "hsmkey/hsm_key_factory.h"
 #include "signconf/signconf_task.h"
@@ -224,7 +224,7 @@ int policy_import(int sockfd, engine_type* engine, db_connection_t *dbconn,
     struct __policy_import_policy* policy2;
     policy_list_t* policy_list;
     const policy_t* policy_walk;
-    zone_list_t* zone_list;
+    zone_list_db_t* zone_list;
     hsm_key_list_t* hsm_key_list;
     int any_update = 0;
 
@@ -787,7 +787,7 @@ int policy_import(int sockfd, engine_type* engine, db_connection_t *dbconn,
             }
         }
     }
-    perform_signconf (sockfd, dbconn, 1);
+    signconf_task_flush_all(engine, dbconn);
 
     if (do_delete) {
         /*
@@ -815,20 +815,20 @@ int policy_import(int sockfd, engine_type* engine, db_connection_t *dbconn,
                  * Check if there are still zones or hsm keys using this policy and
                  * abort if there is
                  */
-                if (!(zone_list = zone_list_new_get_by_policy_id(dbconn, policy_id(policy)))) {
+                if (!(zone_list = zone_list_db_new_get_by_policy_id(dbconn, policy_id(policy)))) {
                     client_printf_err(sockfd, "Unable to check for zones using policy %s from database!\n", policy2->name);
                     policy_free(policy);
                     database_error = 1;
                     continue;
                 }
-                if (zone_list_next(zone_list)) {
-                    zone_list_free(zone_list);
+                if (zone_list_db_next(zone_list)) {
+                    zone_list_db_free(zone_list);
                     client_printf_err(sockfd, "Unable to delete policy %s, there are still zones using this policy!\n", policy2->name);
                     policy_free(policy);
                     database_error = 1;
                     continue;
                 }
-                zone_list_free(zone_list);
+                zone_list_db_free(zone_list);
                 if (!(hsm_key_list = hsm_key_list_new_get_by_policy_id(dbconn, policy_id(policy)))) {
                     client_printf_err(sockfd, "Unable to check for hsm keys using policy %s from database!\n", policy2->name);
                     policy_free(policy);
@@ -886,7 +886,7 @@ int policy_import(int sockfd, engine_type* engine, db_connection_t *dbconn,
         }
     }
 
-    if (any_update) {
+    if (any_update && !engine->config->manual_keygen) {
         hsm_key_factory_schedule_generate_all(engine, 0);
     }
 
