@@ -32,8 +32,8 @@
 #include "config.h"
 #include "adapter/addns.h"
 #include "adapter/adutil.h"
-#include "shared/file.h"
-#include "shared/util.h"
+#include "file.h"
+#include "util.h"
 #include "wire/axfr.h"
 #include "wire/buffer.h"
 #include "wire/edns.h"
@@ -108,8 +108,8 @@ soa_request(query_type* q, engine_type* engine)
         expire = q->zone->xfrd->serial_xfr_acquired;
         expire += ldns_rdf2native_int32(ldns_rr_rdf(rr, SE_SOA_RDATA_EXPIRE));
         if (expire < time_now()) {
-            ods_log_warning("[%s] zone %s expired at %u, and it is now %u: "
-                "not serving soa", axfr_str, q->zone->name, expire, time_now());
+            ods_log_warning("[%s] zone %s expired at %lld, and it is now %lld: "
+                "not serving soa", axfr_str, q->zone->name, (long long)expire, (long long)time_now());
             ldns_rr_free(rr);
             buffer_pkt_set_rcode(q->buffer, LDNS_RCODE_SERVFAIL);
             ods_fclose(fd);
@@ -347,6 +347,7 @@ return_axfr:
     if (q->tcp) {
         ods_log_debug("[%s] return part axfr zone %s", axfr_str,
             q->zone->name);
+        buffer_pkt_set_aa(q->buffer);
         buffer_pkt_set_ancount(q->buffer, total_added);
         buffer_pkt_set_nscount(q->buffer, 0);
         buffer_pkt_set_arcount(q->buffer, 0);
@@ -366,6 +367,7 @@ udp_overflow:
     /* UDP Overflow */
     ods_log_info("[%s] axfr udp overflow zone %s", axfr_str, q->zone->name);
     buffer_set_position(q->buffer, bufpos);
+    buffer_pkt_set_aa(q->buffer);
     buffer_pkt_set_ancount(q->buffer, 1);
     buffer_pkt_set_nscount(q->buffer, 0);
     buffer_pkt_set_arcount(q->buffer, 0);
@@ -551,6 +553,8 @@ ixfr(query_type* q, engine_type* engine)
             } else {
                 ods_log_deeebug("[%s] soa serial %u not found for rr at line %d",
                     axfr_str, q->serial, l);
+                ldns_rr_free(rr);
+                rr = NULL;
                 continue;
             }
         }
